==> mysql-community_8.0.0-dmr.orig.tar.gz: mysql-8.0.0-dmr/sql/dd/info_schema/show_query_builder.h <==
/* Copyright (c) 2016 Oracle and/or its affiliates. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */

#ifndef SQL_DD_SHOW_QUERY_BUILDER_H
#define SQL_DD_SHOW_QUERY_BUILDER_H

#include "m_string.h"                  // C_STRING_WITH_LEN
#include "mysql/mysql_lex_string.h"    // LEX_STRING
#include "parse_tree_nodes.h"          // PT_select_item_list
#include "sql_class.h"                 // THD

namespace dd {
namespace info_schema {

/**
  This class provide framework to build a SELECT_LEX using ParseTree
  nodes.

  Note that this class is designed to help build queries that are
  required to implement SHOW commands over data dictionary tables. It
  does not provide complete framework, e.g., you cannot add a GROUP BY
  node for now, mainly because that is not needed to implement SHOW
  command.

  This class is used by implementation of SHOW command in
  sql/dd/show.cc. The class enables code re-usability.

  One can build SELECT_LEX that represents following,

  ...
    SELECT star_select_item, select_item1, select_item2, ...
      FROM from_item OR FROM PT_derived_table
      WHERE condition AND condition AND ...
      ORDER BY order_by_field1, order_by_field2 , ...
  ...

  Where as, a 'condition' can be one of,
    field_name = "value"
    field_name LIKE "value%"

  One can think of enhancing this framework on need basis.

  Note to server general team: This framework can be used by
  sql/sql_show_status.* implementation. For now, this file is kept
  inside sql/dd, but we can think of moving it out to sql/.

  The memory used while building the this Parse Tree is thd->mem_root.
*/

class Select_lex_builder
{
public:
  Select_lex_builder(const POS *pc, THD *thd)
    :m_pos(pc),
     m_thd(thd),
     m_select_item_list(nullptr),
     m_where_clause(nullptr),
     m_order_by_list(nullptr)
  {
    m_table_reference_list.init(m_thd->mem_root);
  }

  /**
    Add item representing star in "SELECT '*' ...".

    @return false on success.
            true  on failure.
  */

  bool add_star_select_item();


  /**
    Add item representing a column as,
    "SELECT <field_name> AS <alias>, ...".

    The item will be appended to existing list of select items
    for this query.

    @return false on success.
            true  on failure.
  */

  bool add_select_item(const LEX_STRING field_name,
                       const LEX_STRING alias);


  /**
    Add item representing a FROM clause table as,
    "SELECT ... FROM <schema_name>.<table_name> ...".

    Only single table can be added. We cannot build a query with
    JOIN clause for now.

    @return false on success.
            true  on failure.
  */

  bool add_from_item(const LEX_STRING schema_name,
                     const LEX_STRING table_name);


  /**
    Add item representing a FROM clause table as,
    "SELECT ... FROM <sub query or derived table> ...".

    Only single table can be added. We cannot build a query with
    JOIN clause for now.

    @return false on success.
            true  on failure.
  */

  bool add_from_item(PT_derived_table *dt);


  /**
    Prepare item representing a LIKE condition,

    "SELECT ... WHERE <field_name> LIKE <value%> ... "

    This item should be intern added to Select_lex_builder using
    add_condition() method.

    @return pointer to Item* on success.
            nullptr on failure.
  */

  Item *prepare_like_item(const LEX_STRING field_name,
                          const String *wild);


  /**
    Prepare item representing a equal to comparision condition,

    "SELECT ... WHERE <field_name> = <value> ... "

    This item should be intern added to Select_lex_builder using
    add_condition() method.

    @return pointer to Item* on success.
            nullptr on failure.
  */

  Item *prepare_equal_item(const LEX_STRING field_name,
                           const LEX_STRING value);


  /**
    Add a WHERE clause condition to Select_lex_builder.

    "SELECT ... WHERE ... AND <condition> ... "

    If there are existing conditions, then the new condition is
    append to the WHERE clause conditions with a 'AND' condition.

    @return false on success.
            true  on failure.
  */

  bool add_condition(Item *a);


  /**
    Add a ORDER BY clause field to Select_lex_builder.

    "SELECT ... ORDER BY <field_name>, ... "

    If there are existing ORDER BY field, then we append a new
    field to the ORDER BY clause. All the fields are added to be
    order in ascending order.

    @return false on success.
            true  on failure.
  */

  bool add_order_by(const LEX_STRING field_name);


  /**
    This function build ParseTree node that represents this
    Select_lex_builder as sub-query. This enables us to build a
    SELECT_LEX containing a sub-query in its FROM clause. This
    sub-query is represented by ParseTree node PT_derived_table.

    "SELECT ... FROM <PT_dervied_table>, ... "

    @return pointer to PT_derived_table on success.
            nullptr on failure.
  */

  PT_derived_table* prepare_derived_table(const LEX_STRING table_alias);


  /**
    Prepare a SELECT_LEX using all the information information
    added to this Select_lex_builder.

    @return pointer to SELECT_LEX* on success.
            nullptr on failure.
  */

  SELECT_LEX* prepare_select_lex();


private:

  /**
    Prepare a list of expression used to build select items for
    the query being built.

    @return false on success.
            true  on failure.
  */

  template <typename T>
  bool add_to_select_item_list(T *expr)
  {
    // Prepare list if not exist.
    if (!m_select_item_list)
    {
      m_select_item_list= new (m_thd->mem_root) PT_select_item_list();

      if (m_select_item_list == nullptr)
        return true;
    }

    m_select_item_list->push_back(expr);

    return false;
  }


private:
  // Parser current position represented by YYLTYPE
  const POS *m_pos;

  // Current thread
  THD *m_thd;

  // List of select_items for the query.
  PT_select_item_list *m_select_item_list;

  // Table reference in FROM clause for the query.
  Mem_root_array_YY<PT_table_reference *> m_table_reference_list;

  // Expression representing a WHERE clause.
  Item *m_where_clause;

  // List of order by clause elements.
  PT_order_list *m_order_by_list;
};

} // namespace info_schema
} // namespace dd

#endif  /* SQL_DD_SHOW_QUERY_BUILDER_H */

==> mysql-community_8.0.2-dmr.orig.tar.gz: mysql-8.0.2-dmr/sql/dd/info_schema/show_query_builder.h <==
/* Copyright (c) 2016, 2017, Oracle and/or its affiliates. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */

#ifndef SQL_DD_SHOW_QUERY_BUILDER_H
#define SQL_DD_SHOW_QUERY_BUILDER_H

#include "lex_string.h"
#include "mem_root_array.h"
#include "mysql/mysql_lex_string.h"    // LEX_STRING

class Item;
class PT_derived_table;
class PT_order_list;
class PT_select_item_list;
class PT_table_reference;
class SELECT_LEX;
class String;
class THD;
struct YYLTYPE;
typedef YYLTYPE POS;
typedef struct st_mysql_lex_string LEX_STRING;

namespace dd {
namespace info_schema {

/**
  This class provide framework to build a SELECT_LEX using ParseTree
  nodes.

  Note that this class is designed to help build queries that are
  required to implement SHOW commands over data dictionary tables. It
  does not provide complete framework, e.g., you cannot add a GROUP BY
  node for now, mainly because that is not needed to implement SHOW
  command.

  This class is used by implementation of SHOW command in
  sql/dd/show.cc. The class enables code re-usability.

  One can build SELECT_LEX that represents following,

  ...
    SELECT star_select_item, select_item1, select_item2, ...
      FROM from_item OR FROM PT_derived_table
      WHERE condition AND condition AND ...
      ORDER BY order_by_field1, order_by_field2 , ...
  ...

  Where as, a 'condition' can be one of,
    field_name = "value"
    field_name LIKE "value%"

  One can think of enhancing this framework on need basis.

  Note to server general team: This framework can be used by
  sql/sql_show_status.* implementation. For now, this file is kept
  inside sql/dd, but we can think of moving it out to sql/.

  The memory used while building the this Parse Tree is thd->mem_root.
*/

class Select_lex_builder
{
public:
  Select_lex_builder(const POS *pc, THD *thd);

  /**
    Add item representing star in "SELECT '*' ...".

    @return false on success.
            true  on failure.
  */

  bool add_star_select_item();


  /**
    Add item representing a column as,
    "SELECT <field_name> AS <alias>, ...".

    The item will be appended to existing list of select items
    for this query.

    @return false on success.
            true  on failure.
  */

  bool add_select_item(const LEX_STRING field_name,
                       const LEX_STRING alias);


  /**
    Add item representing a FROM clause table as,
    "SELECT ... FROM <schema_name>.<table_name> ...".

    Only single table can be added. We cannot build a query with
    JOIN clause for now.

    @return false on success.
            true  on failure.
  */

  bool add_from_item(const LEX_STRING schema_name,
                     const LEX_STRING table_name);


  /**
    Add item representing a FROM clause table as,
    "SELECT ... FROM <sub query or derived table> ...".

    Only single table can be added. We cannot build a query with
    JOIN clause for now.

    @return false on success.
            true  on failure.
  */

  bool add_from_item(PT_derived_table *dt);


  /**
    Prepare item representing a LIKE condition,

    "SELECT ... WHERE <field_name> LIKE <value%> ... "

    This item should be intern added to Select_lex_builder using
    add_condition() method.

    @return pointer to Item* on success.
            nullptr on failure.
  */

  Item *prepare_like_item(const LEX_STRING field_name,
                          const String *wild);


  /**
    Prepare item representing a equal to comparision condition,

    "SELECT ... WHERE <field_name> = <value> ... "

    This item should be intern added to Select_lex_builder using
    add_condition() method.

    @return pointer to Item* on success.
            nullptr on failure.
  */

  Item *prepare_equal_item(const LEX_STRING field_name,
                           const LEX_STRING value);


  /**
    Add a WHERE clause condition to Select_lex_builder.

    "SELECT ... WHERE ... AND <condition> ... "

    If there are existing conditions, then the new condition is
    append to the WHERE clause conditions with a 'AND' condition.

    @return false on success.
            true  on failure.
  */

  bool add_condition(Item *a);


  /**
    Add a ORDER BY clause field to Select_lex_builder.

    "SELECT ... ORDER BY <field_name>, ... "

    If there are existing ORDER BY field, then we append a new
    field to the ORDER BY clause. All the fields are added to be
    order in ascending order.

    @return false on success.
            true  on failure.
  */

  bool add_order_by(const LEX_STRING field_name);


  /**
    This function build ParseTree node that represents this
    Select_lex_builder as sub-query. This enables us to build a
    SELECT_LEX containing a sub-query in its FROM clause. This
    sub-query is represented by ParseTree node PT_derived_table.

    "SELECT ... FROM <PT_dervied_table>, ... "

    @return pointer to PT_derived_table on success.
            nullptr on failure.
  */

  PT_derived_table* prepare_derived_table(const LEX_STRING table_alias);


  /**
    Prepare a SELECT_LEX using all the information information
    added to this Select_lex_builder.

    @return pointer to SELECT_LEX* on success.
            nullptr on failure.
  */

  SELECT_LEX* prepare_select_lex();


private:

  /**
    Prepare a list of expression used to build select items for
    the query being built.

    @return false on success.
            true  on failure.
  */

  bool add_to_select_item_list(Item *expr);

private:
  // Parser current position represented by YYLTYPE
  const POS *m_pos;

  // Current thread
  THD *m_thd;

  // List of select_items for the query.
  PT_select_item_list *m_select_item_list;

  // Table reference in FROM clause for the query.
  Mem_root_array_YY<PT_table_reference *> m_table_reference_list;

  // Expression representing a WHERE clause.
  Item *m_where_clause;

  // List of order by clause elements.
  PT_order_list *m_order_by_list;
};

} // namespace info_schema
} // namespace dd

#endif  /* SQL_DD_SHOW_QUERY_BUILDER_H */
